@@ -32,6 +32,7 @@ import (
 	is "gotest.tools/v3/assert/cmp"
 
 	"github.com/compose-spec/compose-go/v2/types"
+	"github.com/compose-spec/compose-go/v2/utils"
 )
 
 func buildConfigDetails(yaml string, env map[string]string) types.ConfigDetails {
@@ -453,13 +454,13 @@ func TestParseAndLoad(t *testing.T) {
 
 func TestInvalidTopLevelObjectType(t *testing.T) {
 	_, err := loadYAML("1")
-	assert.ErrorContains(t, err, "Top-level object must be a mapping")
+	assert.ErrorContains(t, err, "top-level object must be a mapping")
 
 	_, err = loadYAML("\"hello\"")
-	assert.ErrorContains(t, err, "Top-level object must be a mapping")
+	assert.ErrorContains(t, err, "top-level object must be a mapping")
 
 	_, err = loadYAML("[\"hello\"]")
-	assert.ErrorContains(t, err, "Top-level object must be a mapping")
+	assert.ErrorContains(t, err, "top-level object must be a mapping")
 }
 
 func TestNonStringKeys(t *testing.T) {
@@ -468,7 +469,7 @@ func TestNonStringKeys(t *testing.T) {
   foo:
     image: busybox
 `)
-	assert.ErrorContains(t, err, "Non-string key at top level: 123")
+	assert.ErrorContains(t, err, "non-string key at top level: 123")
 
 	_, err = loadYAML(`
 services:
@@ -477,7 +478,7 @@ services:
   123:
     image: busybox
 `)
-	assert.ErrorContains(t, err, "Non-string key in services: 123")
+	assert.ErrorContains(t, err, "non-string key in services: 123")
 
 	_, err = loadYAML(`
 services:
@@ -489,7 +490,7 @@ networks:
       config:
         - 123: oh dear
 `)
-	assert.ErrorContains(t, err, "Non-string key in networks.default.ipam.config[0]: 123")
+	assert.ErrorContains(t, err, "non-string key in networks.default.ipam.config[0]: 123")
 
 	_, err = loadYAML(`
 services:
@@ -498,7 +499,7 @@ services:
     environment:
       1: FOO
 `)
-	assert.ErrorContains(t, err, "Non-string key in services.dict-env.environment: 1")
+	assert.ErrorContains(t, err, "non-string key in services.dict-env.environment: 1")
 }
 
 func TestV1Unsupported(t *testing.T) {
@@ -596,8 +597,8 @@ services:
 		"BAR": strPtr("2"),
 		"GA":  strPtr("2.5"),
 		"BU":  strPtr(""),
-		"ZO":  nil,
 		"MEU": strPtr("Shadoks"),
+		"ZO":  nil,
 	}
 
 	assert.Check(t, is.Equal(2, len(config.Services)))
@@ -640,7 +641,7 @@ services:
     environment:
       FOO: ["1"]
 `)
-	assert.ErrorContains(t, err, "services.dict-env.environment.FOO must be a string, number, boolean or null")
+	assert.ErrorContains(t, err, "services.dict-env.environment.FOO must be a boolean, null, number or string")
 }
 
 func TestInvalidEnvironmentObject(t *testing.T) {
@@ -990,7 +991,7 @@ func TestDecodeErrors(t *testing.T) {
 
 	configDetails := buildConfigDetails(dict, nil)
 	_, err := LoadWithContext(context.TODO(), configDetails)
-	assert.Error(t, err, "yaml: line 4: found a tab character that violates indentation")
+	assert.Error(t, err, "failed to parse filename0.yml: yaml: line 4: found a tab character that violates indentation")
 }
 
 func TestBuildProperties(t *testing.T) {
@@ -1054,7 +1055,7 @@ func TestInvalidResource(t *testing.T) {
                 impossible:
                   x: 1
 `)
-	assert.ErrorContains(t, err, "Additional property impossible is not allowed")
+	assert.ErrorContains(t, err, "additional properties 'impossible' not allowed")
 }
 
 func TestInvalidExternalAndDriverCombination(t *testing.T) {
@@ -1196,7 +1197,7 @@ services:
         foo:
           bar: zot
 `)
-	assert.ErrorContains(t, err, "services.tmpfs.volumes.0 Additional property foo is not allowed")
+	assert.ErrorContains(t, err, "services.tmpfs.volumes.0 additional properties 'foo' not allowed")
 }
 
 func TestLoadBindMountSourceMustNotBeEmpty(t *testing.T) {
@@ -1350,7 +1351,7 @@ services:
         tmpfs:
           size: -1
 `)
-	assert.ErrorContains(t, err, "services.tmpfs.volumes.0.tmpfs.size Must be greater than or equal to 0")
+	assert.ErrorContains(t, err, "services.tmpfs.volumes.0.tmpfs.size must be greater than or equal to 0")
 }
 
 func TestLoadTmpfsVolumeSizeMustBeInteger(t *testing.T) {
@@ -2264,7 +2265,7 @@ services:
             - driver: nvidia
               count: 2
 `)
-	assert.ErrorContains(t, err, `capabilities is required`)
+	assert.ErrorContains(t, err, "missing property 'capabilities'")
 }
 
 func TestServiceGpus(t *testing.T) {
@@ -3132,7 +3133,7 @@ services:
 `, nil), func(options *Options) {
 		options.ResolvePaths = false
 	})
-	assert.ErrorContains(t, err, "validating filename0.yml: services.frontend.develop.watch.0 action is required")
+	assert.ErrorContains(t, err, "services.frontend.develop.watch.0 missing property 'action'")
 }
 
 func TestBadServiceConfig(t *testing.T) {
@@ -3729,7 +3730,7 @@ services:
     environment:
       - DEBUG = true
 `)
-	assert.Check(t, strings.Contains(err.Error(), "'services[test].environment': environment variable DEBUG  is declared with a trailing space"))
+	assert.Check(t, strings.Contains(err.Error(), "'services[test].environment' environment variable DEBUG  is declared with a trailing space"), err.Error())
 }
 
 func TestFileModeNumber(t *testing.T) {
@@ -3761,3 +3762,160 @@ services:
 	assert.Equal(t, len(p.Services["test"].Secrets), 1)
 	assert.Equal(t, *p.Services["test"].Secrets[0].Mode, types.FileMode(0o440))
 }
+
+func TestServiceProvider(t *testing.T) {
+	p, err := loadYAML(`
+name: service-provider
+services:
+  test:
+    provider:
+      type: foo
+      options:
+        bar: zot
+        strings:
+          - foo
+          - bar
+        numbers:
+          - 12
+          - 34
+        booleans:
+          - true
+          - false
+`)
+	assert.NilError(t, err)
+	assert.DeepEqual(t, p.Services["test"].Provider, &types.ServiceProviderConfig{
+		Type: "foo",
+		Options: types.MultiOptions{
+			"bar":      []string{"zot"},
+			"strings":  []string{"foo", "bar"},
+			"numbers":  []string{"12", "34"},
+			"booleans": []string{"true", "false"},
+		},
+	})
+
+	_, err = loadYAML(`
+name: service-provider
+services:
+  test:
+    provider:
+      options:
+        bar: zot
+        strings: foo
+        numbers: 12
+        booleans: true
+`)
+	assert.ErrorContains(t, err, "services.test.provider missing property 'type'")
+}
+
+func TestImageVolume(t *testing.T) {
+	p, err := loadYAML(`
+name: imageVolume
+services:
+  test:
+    volumes:
+      - type: image
+        source: app/image
+        target: /mnt/image
+        image:
+          subpath: /foo
+`)
+	assert.NilError(t, err)
+	assert.DeepEqual(t, p.Services["test"].Volumes[0], types.ServiceVolumeConfig{
+		Type:   "image",
+		Source: "app/image",
+		Target: "/mnt/image",
+		Image:  &types.ServiceVolumeImage{SubPath: "/foo"},
+	})
+}
+
+func TestNpipeVolume(t *testing.T) {
+	p, err := loadYAML(`
+name: imageVolume
+services:
+  test:
+    volumes:
+      - type: npipe
+        source: \\.\pipe\docker_engine
+        target: \\.\pipe\docker_engine
+        image:
+          subpath: /foo
+`)
+	assert.NilError(t, err)
+	assert.DeepEqual(t, p.Services["test"].Volumes[0], types.ServiceVolumeConfig{
+		Type:   "npipe",
+		Source: "\\\\.\\pipe\\docker_engine",
+		Target: "\\\\.\\pipe\\docker_engine",
+		Image:  &types.ServiceVolumeImage{SubPath: "/foo"},
+	})
+}
+
+func TestInterfaceName(t *testing.T) {
+	p, err := loadYAML(`
+name: interface-name
+services:
+  test:
+    networks:
+      test:
+        interface_name: eth0
+`)
+	assert.NilError(t, err)
+	assert.DeepEqual(t, p.Services["test"].Networks["test"], &types.ServiceNetworkConfig{
+		InterfaceName: "eth0",
+	})
+}
+
+func TestModel(t *testing.T) {
+	p, err := loadYAML(`
+name: model
+services:
+  test_array:
+    models:
+      - foo
+
+  test_mapping:
+    models:
+      foo:
+        endpoint_var: MODEL_URL
+        model_var: MODEL
+
+models:
+  foo:
+    model: ai/model
+    context_size: 1024
+    runtime_flags: 
+      - "--some-flag"
+`)
+	assert.NilError(t, err)
+	assert.DeepEqual(t, p.Models["foo"], types.ModelConfig{
+		Model:        "ai/model",
+		ContextSize:  1024,
+		RuntimeFlags: []string{"--some-flag"},
+	})
+	assert.DeepEqual(t, p.Services["test_array"].Models, map[string]*types.ServiceModelConfig{
+		"foo": nil,
+	})
+	assert.DeepEqual(t, p.Services["test_mapping"].Models, map[string]*types.ServiceModelConfig{
+		"foo": {
+			EndpointVariable: "MODEL_URL",
+			ModelVariable:    "MODEL",
+		},
+	})
+	assert.DeepEqual(t, p.ModelNames(), []string{"foo"})
+	assert.Check(t, utils.ArrayContains(p.ServicesWithModels(), []string{"test_array", "test_mapping"}), p.ServicesWithModels())
+}
+
+func TestAttestations(t *testing.T) {
+	p, err := loadYAML(`
+name: attestations
+services:
+  test:
+    build:
+      context: .
+      provenance: mode=max
+      sbom: true
+`)
+	assert.NilError(t, err)
+	build := p.Services["test"].Build
+	assert.Equal(t, build.Provenance, "mode=max")
+	assert.Equal(t, build.SBOM, "true")
+}
